#!/usr/bin/env python3
import sys
if sys.version_info < (3, 7):
    raise RuntimeError('At least Python 3.7 is required')

import os, yaml, sys
from pathlib import Path

try:
    from yaml import CLoader as Loader
except ImportError:
    from yaml import Loader

def qemu(*args):
    os.execlp('qemu-system-x86_64', 'qemu-system-x86_64', *(str(x) for x in args))


def qemu_keyed(**kwargs):
    args = ['-enable-kvm']
    for key, value in kwargs.items():
        if isinstance(value, (list, tuple, set)):
            for subvalue in value:
                args.append(f'-{key}')
                args.append(subvalue)

        elif value is not None:
            args.append(f'-{key}')
            args.append(value)
        else:
            args.append(f'-{key}')

    qemu('-enable-kvm', *args)



ROOT = Path(__file__).parent
if __name__ == '__main__':
    from argparse import ArgumentParser

    aparser = ArgumentParser(usage = '%(prog)s -c config.yml')
    aparser.add_argument('--config', '-c', dest = 'config', type = Path, required = True, help = 'Configuration YAML.')
    args = aparser.parse_args()

    if not args.config.is_absolute():
        args.config = (ROOT / args.config).resolve()

    monitor = f'{args.config}.socket'
    config = {
        'monitor' : f'unix:{monitor},server,nowait'
    }
    with args.config.open('rt', encoding = 'utf8') as inf:
        config.update(yaml.load(inf, Loader = Loader))

    print(f'Listening on {monitor}', file = sys.stderr)
    qemu_keyed(**config)
